-- Navajo_Decisions
-- Author: Jarcast
--=======================================================================================================================
-- INCLUDES
--=======================================================================================================================
include("FLuaVector.lua")
include("PlotIterators.lua")
--=======================================================================================================================
-- GLOBALS
--=======================================================================================================================
------------------------------------------------------------------------------------------------------------------------
local g_ConvertTextKey  		= Locale.ConvertTextKey
local g_MathCeil				= math.ceil
local g_MathFloor				= math.floor
local g_MathMax					= math.max
local g_MathMin					= math.min
				
local Players 					= Players
local Teams 					= Teams

local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--=======================================================================================================================
-- UTILITIES
--=======================================================================================================================
-- UTILITIES
--------------------------------------------------------------------------------------------------------------------------
--Game_GetRound
function Game_GetRound(num, idp)
	local mult = 10^(idp or 0)
	return g_MathFloor(num * mult + 0.5) / mult
end
local g_GetRound = Game_GetRound
------------------------------------------------------------------------------------------------------------------------
--Player_SendWorldEvent
local notificationWorldEventID = NotificationTypes["NOTIFICATION_DIPLOMACY_DECLARATION"]
function Player_SendWorldEvent(pPlayer, description, includeHuman)
	print("Sending World Event: ", description)
	local pPlayerTeam = Teams[pPlayer:GetTeam()]
	if (not includeHuman) and pPlayer:IsHuman() then return end
	if (not pPlayerTeam:IsHasMet(activeTeamID)) then return end
	activePlayer:AddNotification(notificationWorldEventID, description, "[COLOR_POSITIVE_TEXT]World Events[ENDCOLOR]", -1, -1)
end 
-------------------------------------------------------------------------------------------------------------------------
--g_JFD_GlobalUserSettings_Table
local g_JFD_GlobalUserSettings_Table = {}
for row in DB.Query("SELECT Type, Value FROM JFD_GlobalUserSettings;") do 	
	g_JFD_GlobalUserSettings_Table[row.Type] = row.Value
end

--Game_GetUserSetting
function Game_GetUserSetting(type)
	return g_JFD_GlobalUserSettings_Table[type]
end
--=======================================================================================================================
-- MOD USE
--=======================================================================================================================
-- MODS
-------------------------------------------------------------------------------------------------------------------------
--IsCPDLL
function Game_IsCPActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "d1b6328c-ff44-4b0d-aad7-c657f83610cd" then
			return true
		end
	end
	return false
end
local isUsingCBP = Game_IsCPActive()

function Game_IsMoreLuxuriesActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "be2d13ee-d18d-410e-8f04-7cd19cd99a98" then
			return true
		end
	end
	return false
end
local isUsingMoreLuxuries = Game_IsMoreLuxuriesActive()
-------------------------------------------------------------------------------------------------------------------------
-- SETTINGS
-------------------------------------------------------------------------------------------------------------------------
local userSettingMoreUC	= (Game_GetUserSetting("JAR_NAVAJO_4UC") == 1)
--=======================================================================================================================
-- DECISIONS
--=======================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local iCiv     = GameInfoTypes["CIVILIZATION_JAR_NAVAJO"]
local techEconomicsID 	 = GameInfoTypes.TECH_ECONOMICS
local eraMedievalID 	 = GameInfoTypes["ERA_MEDIEVAL"]
local resourceCaptainsID = GameInfoTypes["RESOURCE_JFD_CAPTAINS"]
local policyDecisionsNavajoRaidNeighborsID = GameInfoTypes["POLICY_DECISIONS_NAVAJO_RAID_NEIGHBORS"]
local policyDecisionsNavajoPromoteDiyogiID = GameInfoTypes["POLICY_DECISIONS_NAVAJO_PROMOTE_DIYOGI"]
local policyDecisionsNavajoPromoteSmithingID = GameInfoTypes["POLICY_DECISIONS_NAVAJO_PROMOTE_SMITHING"]
local JewelryID = GameInfoTypes["RESOURCE_JEWELRY"]
local BuildingSmithing	= GameInfoTypes["BUILDING_PROMOTE_SMITHING"]
local DummySmithing		= GameInfoTypes["BUILDING_DUMMY_SMITHING"]
local tSmithingResources = {}
tSmithingResources[GameInfoTypes.RESOURCE_COPPER] = true
tSmithingResources[GameInfoTypes.RESOURCE_GEMS] = true
tSmithingResources[GameInfoTypes.RESOURCE_GOLD] = true
tSmithingResources[GameInfoTypes.RESOURCE_PEARLS] = true
tSmithingResources[GameInfoTypes.RESOURCE_SILVER] = true
if isUsingMoreLuxuries then
tSmithingResources[GameInfoTypes.RESOURCE_AMBER] = true
tSmithingResources[GameInfoTypes.RESOURCE_JADE] = true
tSmithingResources[GameInfoTypes.RESOURCE_LAPIS] = true
end
local barracksID    = GameInfoTypes["BUILDING_BARRACKS"]
local armoryID = GameInfoTypes["BUILDING_ARMORY"]
local RaidPromoID = GameInfoTypes["PROMOTION_RAID_NEIGHBORS"]


-------------------------------------------------------------------------------------------------------------------------
-- Navajo: Organize raids against neighbors
-------------------------------------------------------------------------------------------------------------------------
local Decisions_Jarcast_Navajo_RaidNeighbors = {}
	Decisions_Jarcast_Navajo_RaidNeighbors.Name = "TXT_KEY_DECISIONS_JAR_NAVAJO_RAID_NEIGHBORS"
	Decisions_Jarcast_Navajo_RaidNeighbors.Desc = "TXT_KEY_DECISIONS_JAR_NAVAJO_RAID_NEIGHBORS_DESC"
	HookDecisionCivilizationIcon(Decisions_Jarcast_Navajo_RaidNeighbors, "CIVILIZATION_JAR_NAVAJO")
	Decisions_Jarcast_Navajo_RaidNeighbors.CanFunc = (
	function(pPlayer)
		if pPlayer:GetCivilizationType() ~= iCiv then return false, false end
		if pPlayer:HasPolicy(policyDecisionsNavajoRaidNeighborsID) then
			Decisions_Jarcast_Navajo_RaidNeighbors.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_RAID_NEIGHBORS_ENACTED_DESC")
			return false, false, true 
		end
		
		local req1 = false
		local reqIcon1 = "[ICON_JARUNCHECKBOX]"
		if pPlayer:GetCurrentEra() >= eraMedievalID then
			req1 = true
			reqIcon1 = "[ICON_CHECKBOX]"
		end
		
		local capital = pPlayer:GetCapitalCity()
		if (not capital) then return false, false end
		
		local req2 = false
		local reqIcon2 = "[ICON_JARUNCHECKBOX]"
		if capital:IsHasBuilding(barracksID) and capital:IsHasBuilding(armoryID) then
			req2 = true
			reqIcon2 = "[ICON_CHECKBOX]"
		end
		
		local req3 = false
		local reqIcon3 = "[ICON_JARUNCHECKBOX]"
		--Decisions_Jarcast_Navajo_RaidNeighbors.Data1 = nil
		for city in pPlayer:Cities() do
			local plot = city:Plot()
			local iNumUnits = plot:GetNumUnits()
			for iVal = 0,(iNumUnits - 1) do
				local unit = plot:GetUnit(iVal)
				if unit:GetUnitClassType() == GameInfo.UnitClasses.UNITCLASS_GREAT_GENERAL.ID then
					--Decisions_Jarcast_Navajo_RaidNeighbors.Data1 = unit
					req3 = true
					reqIcon3 = "[ICON_CHECKBOX]"
					break
				end
			end
		end
		
		local costGold = g_GetRound(300*iMod)
		local resourceID = iMagistrate
		local strResource = "[ICON_MAGISTRATES] Magistrates"
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
			strResource = "[ICON_JFD_CAPTAINS] Captains"
		end
		Decisions_Jarcast_Navajo_RaidNeighbors.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_RAID_NEIGHBORS_DESC", reqIcon1, reqIcon2, reqIcon3, costGold, strResource)	
		
		if (not req1) then return true, false end
		if (not req2) then return true, false end
		if (not req3) then return true, false end
		if pPlayer:GetGold() < costGold then return true, false end
		if pPlayer:GetNumResourceAvailable(resourceID, false) < 2 then return true, false end
		return true, true
	end
	)
	
	Decisions_Jarcast_Navajo_RaidNeighbors.DoFunc = (
	function(pPlayer)
		local resourceID = iMagistrate
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
		end
		local numCities = pPlayer:GetNumCities()
		local costGold = g_GetRound(300*iMod)
		pPlayer:ChangeNumResourceTotal(resourceID, -2)
		pPlayer:ChangeGold(-costGold)
		--pUnit:Kill()
		for city in pPlayer:Cities() do
			local pPlot = city:Plot()
			newunit = pPlayer:InitUnit(GameInfoTypes["UNIT_JAR_NAABAAHII"], pPlot:GetX(), pPlot:GetY() )
			newunit:SetExperience(city:GetDomainFreeExperience( newunit:GetDomainType() )) 
			newunit:JumpToNearestValidPlot();
		end
		if isUsingCBP then
			pPlayer:GrantPolicy(policyDecisionsNavajoRaidNeighborsID, true)
		else
			pPlayer:SetNumFreePolicies(1)
			pPlayer:SetNumFreePolicies(0)
			pPlayer:SetHasPolicy(policyDecisionsNavajoRaidNeighborsID, true)
		end
	end
	)

Decisions_AddCivilisationSpecific(iCiv, "Decisions_Jarcast_Navajo_RaidNeighbors", Decisions_Jarcast_Navajo_RaidNeighbors)
--=======================================================================================================================
function NavajoDecisionRaidPromotion(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		for pUnit in pPlayer:Units() do
			local unitplot = pUnit:GetPlot()
			if pUnit:IsHasPromotion(RaidPromoID) and unitplot:GetOwner() ~= iPlayer then
				if unitplot:GetImprovementType() ~= -1 and unitplot:IsImprovementPillaged() then
					local goldgain = math.floor( pUnit:GetExperience()*0.10 )
					pPlayer:ChangeGold(goldgain)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "+" .. goldgain .. " [ICON_GOLD]", 2)
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then 
GameEvents.PlayerDoTurn.Add(NavajoDecisionRaidPromotion)
end

if not userSettingMoreUC then
-------------------------------------------------------------------------------------------------------------------------
-- Navajo: Promote diyogi weaving
-------------------------------------------------------------------------------------------------------------------------
local Decisions_Jarcast_Navajo_PromoteDiyogi = {}
	Decisions_Jarcast_Navajo_PromoteDiyogi.Name = "TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_DIYOGI"
	Decisions_Jarcast_Navajo_PromoteDiyogi.Desc = "TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_DIYOGI_DESC"
	HookDecisionCivilizationIcon(Decisions_Jarcast_Navajo_PromoteDiyogi, "CIVILIZATION_JAR_NAVAJO")
	Decisions_Jarcast_Navajo_PromoteDiyogi.CanFunc = (
	function(pPlayer)
		if pPlayer:GetCivilizationType() ~= iCiv then return false, false end
		if pPlayer:HasPolicy(policyDecisionsNavajoPromoteDiyogiID) then
			Decisions_Jarcast_Navajo_PromoteDiyogi.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_DIYOGI_ENACTED_DESC")
			return false, false, true 
		end
		
		local req1 = false
		local reqIcon1 = "[ICON_JARUNCHECKBOX]"
		if Teams[pPlayer:GetTeam()]:IsHasTech(techEconomicsID) then
			req1 = true
			reqIcon1 = "[ICON_CHECKBOX]"
		end
				
		local costGold = g_GetRound((400)*iMod)
		local resourceID = iMagistrate
		local strResource = "[ICON_MAGISTRATES] Magistrates"
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
			strResource = "[ICON_JFD_CAPTAINS] Captains"
		end
		Decisions_Jarcast_Navajo_PromoteDiyogi.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_DIYOGI_DESC", reqIcon1, costGold, strResource)	
		
		if (not req1) then return true, false end
		if pPlayer:GetGold() < costGold then return true, false end
		if pPlayer:GetNumResourceAvailable(resourceID, false) < 1 then return true, false end
		return true, true
	end
	)
	
	Decisions_Jarcast_Navajo_PromoteDiyogi.DoFunc = (
	function(pPlayer)
		local costGold = g_GetRound(500*iMod)
		pPlayer:ChangeGold(-costGold)
		local resourceID = iMagistrate
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
		end
		pPlayer:ChangeNumResourceTotal(resourceID, -1)
		if isUsingCBP then
			pPlayer:GrantPolicy(policyDecisionsNavajoPromoteDiyogiID, true)
		else
			pPlayer:SetNumFreePolicies(1)
			pPlayer:SetNumFreePolicies(0)
			pPlayer:SetHasPolicy(policyDecisionsNavajoPromoteDiyogiID, true)
		end
		Player_SendWorldEvent(pPlayer, g_ConvertTextKey("TXT_KEY_WORLD_EVENT_JAR_NAVAJO_PROMOTE_DIYOGI", pPlayer:GetName(), pPlayer:GetCivilizationShortDescription())) 
	end
	)

Decisions_AddCivilisationSpecific(iCiv, "Decisions_Jarcast_Navajo_PromoteDiyogi", Decisions_Jarcast_Navajo_PromoteDiyogi)

else
-------------------------------------------------------------------------------------------------------------------------
-- Navajo: Promote metal smithing (active when additional unique components are enabled)
-------------------------------------------------------------------------------------------------------------------------
local Decisions_Jarcast_Navajo_PromoteSmithing = {}
	Decisions_Jarcast_Navajo_PromoteSmithing.Name = "TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_SMITHING"
	Decisions_Jarcast_Navajo_PromoteSmithing.Desc = "TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_SMITHING_DESC"
	HookDecisionCivilizationIcon(Decisions_Jarcast_Navajo_PromoteSmithing, "CIVILIZATION_JAR_NAVAJO")
	Decisions_Jarcast_Navajo_PromoteSmithing.CanFunc = (
	function(pPlayer)
		if pPlayer:GetCivilizationType() ~= iCiv then return false, false end
		if pPlayer:HasPolicy(policyDecisionsNavajoPromoteSmithingID) then
			Decisions_Jarcast_Navajo_PromoteSmithing.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_SMITHING_ENACTED_DESC")
			return false, false, true 
		end
		
		local req1 = false
		local reqIcon1 = "[ICON_JARUNCHECKBOX]"
		if Teams[pPlayer:GetTeam()]:IsHasTech(techEconomicsID) then
			req1 = true
			reqIcon1 = "[ICON_CHECKBOX]"
		end
				
		local costGold = g_GetRound((300)*iMod)
		local resourceID = iMagistrate
		local strResource = "[ICON_MAGISTRATES] Magistrates"
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
			strResource = "[ICON_JFD_CAPTAINS] Captains"
		end
		Decisions_Jarcast_Navajo_PromoteSmithing.Desc = g_ConvertTextKey("TXT_KEY_DECISIONS_JAR_NAVAJO_PROMOTE_SMITHING_DESC", reqIcon1, costGold, strResource)	
		
		if (not req1) then return true, false end
		if pPlayer:GetGold() < costGold then return true, false end
		if pPlayer:GetNumResourceAvailable(resourceID, false) < 1 then return true, false end
		return true, true
	end
	)
	
	Decisions_Jarcast_Navajo_PromoteSmithing.DoFunc = (
	function(pPlayer)
		local costGold = g_GetRound((300)*iMod)
		pPlayer:ChangeGold(-costGold)
		local resourceID = iMagistrate
		if resourceCaptainsID then
			resourceID = resourceCaptainsID
		end
		pPlayer:ChangeNumResourceTotal(resourceID, -1)
		if isUsingCBP then
			pPlayer:GrantPolicy(policyDecisionsNavajoPromoteSmithingID, true)
		else
			pPlayer:SetNumFreePolicies(1)
			pPlayer:SetNumFreePolicies(0)
			pPlayer:SetHasPolicy(policyDecisionsNavajoPromoteSmithingID, true)
		end
		pPlayer:ChangeNumResourceTotal( JewelryID , 2);
		-- loop mania begins
		for pCity in pPlayer:Cities() do
			pCity:SetNumRealBuilding( BuildingSmithing , 1);
			
			local pPlot = pCity:Plot()
			local smithingcount = 0
			for adjacentPlot in PlotAreaSweepIterator(pPlot, 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
				if Players[adjacentPlot:GetOwner()] == pPlayer then
					for k, v in pairs(tSmithingResources) do
						if (adjacentPlot:GetResourceType() == k) then
							smithingcount = smithingcount +1
						end
					end
				end
			end
			pCity:SetNumRealBuilding( DummySmithing , smithingcount);
		end
		--Player_SendWorldEvent(pPlayer, g_ConvertTextKey("TXT_KEY_WORLD_EVENT_JAR_NAVAJO_PROMOTE_SMITHING", pPlayer:GetName(), pPlayer:GetCivilizationShortDescription())) 
	end
	)

	Decisions_Jarcast_Navajo_PromoteSmithing.Monitors = {}
	Decisions_Jarcast_Navajo_PromoteSmithing.Monitors[GameEvents.CityBoughtPlot] = (	
	function(playerID, cityID, plotX, plotY, bGold, bFaithOrCulture)
		local pPlot = Map.GetPlot(plotX, plotY)
		local pCity = pPlot:GetWorkingCity()
		local pPlayer = Players[playerID]
		if pPlayer:HasPolicy(policyDecisionsNavajoPromoteSmithingID) then
			for k, v in pairs(tSmithingResources) do
				if (pPlot:GetResourceType() == k) then
					if pCity:IsHasBuilding(DummySmithing) then
						local count = pCity:GetNumBuilding(DummySmithing)
						pCity:SetNumRealBuilding( DummySmithing , count +1)
					end
				end
			end
		end
	end
	)

Decisions_AddCivilisationSpecific(iCiv, "Decisions_Jarcast_Navajo_PromoteSmithing", Decisions_Jarcast_Navajo_PromoteSmithing)
end
--=======================================================================================================================
print("Jarcast's Navajo Decisions: loaded")
--=======================================================================================================================
--=======================================================================================================================
