-- Jarcast Navajo Functions
-- Author: Jarcast
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
include("FLuaVector.lua")
include("PlotIterators.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local g_ConvertTextKey  		= Locale.ConvertTextKey
local g_MapGetPlot				= Map.GetPlot
local g_MathCeil				= math.ceil
local g_MathFloor				= math.floor
local g_MathMax					= math.max
local g_MathMin					= math.min
				
local Players 					= Players
local HexToWorld 				= HexToWorld
local ToHexFromGrid 			= ToHexFromGrid
local Teams 					= Teams

local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]

local iCiv = GameInfoTypes["CIVILIZATION_JAR_NAVAJO"]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
function JFD_IsCivilisationActive(civilisationID)
	for iSlot = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotStatus = PreGame.GetSlotStatus(iSlot)
		if (slotStatus == SlotStatus["SS_TAKEN"] or slotStatus == SlotStatus["SS_COMPUTER"]) then
			if PreGame.GetCivilization(iSlot) == civilisationID then
				return true
			end
		end
	end

	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function C15_IsPlayerAtWarWithAnyone(playerID, bIncludeMinors)
    local pPlayer = Players[playerID]
    local iTeam = pPlayer:GetTeam()
    local pTeam = Teams[iTeam]
    local bIsAtWar = false
    for i = 0, GameDefines.MAX_CIV_PLAYERS - 1 do -- Iterate through everyone
        if i ~= playerID then -- If this isn't the Player, since you can't be at war with yourself...
            local pOtherPlayer = Players[i]
            if pOtherPlayer:IsAlive() then -- Can't be at war with the dead :p
                if not pOtherPlayer:IsBarbarian() then -- You're always at war with Barbs so ignore them
                    if (bIncludeMinors) or ((not bIncludeMinors) and (not pOtherPlayer:IsMinorCiv())) then -- If you've said to include City-States, then everyone will qualify; if you've said to not include them, then make sure pOtherPlayer isn't a City-State
                        local iOtherTeam = pOtherPlayer:GetTeam()
                        if pTeam:IsAtWar(iOtherTeam) then -- The crucial Method
                            bIsAtWar = true
                            break -- We've found someone we're at war with, so we don't need to check for any longer
                        end
                    end
                end
            end
        end
    end
    return bIsAtWar -- Returns either true or false to wherever you called the function
end
-------------------------------------------------------------------------------------------------------------------------
function Civ1HasOpenBorderFromCiv2(iPlayer1, iPlayer2)
	if Teams[Players[iPlayer2]:GetTeam()]:IsAllowsOpenBordersToTeam( Players[iPlayer1]:GetTeam() ) then
        return true
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
-------------------------------------------------------------------------------------------------------------------------
-- Counts all the great works of music owned by a civ
local tBuildingGWMusic = {}
for row in DB.Query("SELECT a.ID buildingID, b.ID classID FROM Buildings a, BuildingClasses b WHERE a.BuildingClass = b.Type AND a.GreatWorkSlotType = 'GREAT_WORK_SLOT_MUSIC'") do
	tBuildingGWMusic[row.buildingID] = row.classID
end

function GetNumGreatWorkMusic(pPlayer) 
	local GWMcount = 0
	if pPlayer:IsAlive() then
		for city in pPlayer:Cities() do
			for k,v in pairs(tBuildingGWMusic) do
				if city:IsHasBuilding(k) then
					GWMcount= GWMcount + city:GetNumGreatWorksInBuilding(v)
				end
			end
		end
	end
	return GWMcount
end
-------------------------------------------------------------------------------------------------------------------------
function NavajoHasTech(techID)
	local bNavHasTech = false
	for iMajPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
		local pPlayer = Players[iMajPlayer]
		if pPlayer:GetCivilizationType() == iCiv and (Teams[pPlayer:GetTeam()]:IsHasTech( techID )) then
			bNavHasTech = true
		end
	end
	return bNavHasTech
end
-------------------------------------------------------------------------------------------------------------------------
function PlayersShareWarEnemy(playerAID, playerBID)
    local pPlayerA = Players[playerAID]
	local pPlayerB = Players[playerBID]
    local iTeamA = pPlayerA:GetTeam()
    local pTeamA = Teams[iTeamA]
	local iTeamB = pPlayerB:GetTeam()
    local pTeamB = Teams[iTeamB]
	local bAlliesAtWar = false
    for i = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
        if i ~= playerAID and i ~= playerBID then
            local pOtherPlayer = Players[i]
            if pOtherPlayer:IsAlive() then
				local iOtherTeam = pOtherPlayer:GetTeam()
				if pTeamA:IsAtWar(iOtherTeam) and pTeamB:IsAtWar(iOtherTeam) then
					bAlliesAtWar = true
					break
                end
            end
        end
    end
    return bAlliesAtWar
end
-------------------------------------------------------------------------------------------------------------------------
-- base converter
local floor,insert = math.floor, table.insert
function basen(n,b)
    n = floor(n)
    if not b or b == 10 then return tostring(n) end
    local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    local t = {}
    local sign = ""
    if n < 0 then
        sign = "-"
    n = -n
    end
    repeat
        local d = (n % b) + 1
        n = floor(n / b)
        insert(t, 1, digits:sub(d,d))
    until n == 0
    return sign .. table.concat(t,"")
end

-- Tourism dummies
local maxpower = 9
tTourismDummies = {}
for power=0,maxpower do
	tTourismDummies[power] = GameInfoTypes["BUILDING_JAR_DUMMY_TOURISM_" .. tostring(power)]
end


function Jarcast_SetBinaryTourismBuildings( pCity, integer)
    local binary = basen(integer,2)
    local numcifre = string.len(tostring(binary))
    print("Tourism Bonus= ",integer, "(dec) =", binary, "(bin)")
    for i=-numcifre,-1 do
        if string.len(tostring(binary)) == -i and binary~=0 then
            --print(-i)
            pCity:SetNumRealBuilding(tTourismDummies[-i-1], 1)
            print("Given BUILDING_JAR_DUMMY_TOURISM_", -i-1, "=", floor(math.pow(2,-i-1)), " tourism.")
            binary = floor( binary - math.pow(10, -i-1) )
            --print(binary)
        else
            pCity:SetNumRealBuilding(tTourismDummies[-i-1], 0)
            print("Removed BUILDING_JAR_DUMMY_TOURISM_", -i-1)
        end
    end
end
--=======================================================================================================================
-- MOD USE
--=======================================================================================================================
-- MODS
-------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------
-- SETTINGS
-------------------------------------------------------------------------------------------------------------------------

--==========================================================================================================================
-- GAME DEFINES
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iFood = GameInfo.Yields.YIELD_FOOD.ID
local traitNavajoID = GameInfoTypes["TRAIT_JAR_NAVAJO"]
local iEnemywayDummy = GameInfoTypes["BUILDING_JAR_BLESSINGDM5"]
local iTimerDummy = GameInfoTypes["BUILDING_JAR_TIMER"]
local iHappyFaithDummy = GameInfoTypes["BUILDING_JAR_GWMUSIC"]
local prophetcount = 0
local CodeTalkerID = GameInfoTypes.UNIT_JAR_CODETALKER
--for row in DB.Query("SELECT PrereqTech FROM Units WHERE Type='UNIT_JAR_CODETALKER'") do local iTechreq = row.PrereqTech end
local iTechreq = 0
for row in DB.Query("SELECT a.PrereqTech Prereqtech, b.ID techID FROM Units a, Technologies b WHERE a.PrereqTech = b.Type AND a.Type='UNIT_JAR_CODETALKER'") do
	iTechreq = row.techID
end
local CodeTalkerPromoID = GameInfoTypes["PROMOTION_JAR_CODETALKER"]
local CTBonusPromoID = GameInfoTypes["PROMOTION_JAR_CODETALKERPOS"]
local CTMalusPromoID = GameInfoTypes["PROMOTION_JAR_CODETALKERNEG"]
--------
local hoganID = GameInfo.Improvements.IMPROVEMENT_JAR_HOGAN.ID
local hoganBuilding1 =  GameInfoTypes["BUILDING_JAR_HOGAN1"]
local hoganBuilding2 =  GameInfoTypes["BUILDING_JAR_HOGAN2"]
local hoganBuilding3 =  GameInfoTypes["BUILDING_JAR_HOGAN3"]
local hoganBuilding4 =  GameInfoTypes["BUILDING_JAR_HOGAN4"]
local blessprod2 =  	GameInfoTypes["BUILDING_JAR_BLESSPROD2"]
local blessprod =  		GameInfoTypes["BUILDING_JAR_BLESSPROD"]
local blessfood =  		GameInfoTypes["BUILDING_JAR_BLESSFOOD"]
local blessbeautydum =  GameInfoTypes["BUILDING_JAR_BLESSBEAUTY"]
local blessgoldmod =  	GameInfoTypes["BUILDING_JAR_BLESSGOLD"]
local blessgold2 =  	GameInfoTypes["BUILDING_JAR_BLESSGOLD2"]
local blessscimod =  	GameInfoTypes["BUILDING_JAR_BLESSSCI"]
local blesssci2 =  		GameInfoTypes["BUILDING_JAR_BLESSSCI2"]
local blessfaith =  	GameInfoTypes["BUILDING_JAR_BLESSFAITH"]
--------
local blessing1 = GameInfoTypes["BUILDING_JAR_BLESSING1"]
local blessing2 = GameInfoTypes["BUILDING_JAR_BLESSING2"]
local blessing3 = GameInfoTypes["BUILDING_JAR_BLESSING3"]
local blessing4 = GameInfoTypes["BUILDING_JAR_BLESSING4"]
local blessing5 = GameInfoTypes["BUILDING_JAR_BLESSING5"]
local blessing6 = GameInfoTypes["BUILDING_JAR_BLESSING6"]
local blessing7 = GameInfoTypes["BUILDING_JAR_BLESSING7"]
local tDumBlessings = {}
tDumBlessings[blessing1] = true
tDumBlessings[blessing2] = true
tDumBlessings[blessing3] = true
tDumBlessings[blessing4] = true
tDumBlessings[blessing5] = true
tDumBlessings[blessing6] = true
tDumBlessings[blessing7] = true
--
local blessing1A = GameInfoTypes["BUILDING_JAR_BLESSING1A"]
local blessing2A = GameInfoTypes["BUILDING_JAR_BLESSING2A"]
local blessing3A = GameInfoTypes["BUILDING_JAR_BLESSING3A"]
local blessing4A = GameInfoTypes["BUILDING_JAR_BLESSING4A"]
local blessing5A = GameInfoTypes["BUILDING_JAR_BLESSING5A"]
local blessing6A = GameInfoTypes["BUILDING_JAR_BLESSING6A"]
local blessing7A = GameInfoTypes["BUILDING_JAR_BLESSING7A"]
local tDumBlessingsA = {}
tDumBlessingsA[blessing1A] = true
tDumBlessingsA[blessing2A] = true
tDumBlessingsA[blessing3A] = true
tDumBlessingsA[blessing4A] = true
tDumBlessingsA[blessing5A] = true
tDumBlessingsA[blessing6A] = true
tDumBlessingsA[blessing7A] = true
--
local blessing1B = GameInfoTypes["BUILDING_JAR_BLESSING1B"]
local blessing2B = GameInfoTypes["BUILDING_JAR_BLESSING2B"]
local blessing3B = GameInfoTypes["BUILDING_JAR_BLESSING3B"]
local blessing4B = GameInfoTypes["BUILDING_JAR_BLESSING4B"]
local blessing5B = GameInfoTypes["BUILDING_JAR_BLESSING5B"]
local blessing6B = GameInfoTypes["BUILDING_JAR_BLESSING6B"]
local blessing7B = GameInfoTypes["BUILDING_JAR_BLESSING7B"]
local tDumBlessingsB = {}
tDumBlessingsB[blessing1B] = true
tDumBlessingsB[blessing2B] = true
tDumBlessingsB[blessing3B] = true
tDumBlessingsB[blessing4B] = true
tDumBlessingsB[blessing5B] = true
tDumBlessingsB[blessing6B] = true
tDumBlessingsB[blessing7B] = true
--
local blessing1C = GameInfoTypes["BUILDING_JAR_BLESSING1C"]
local blessing2C = GameInfoTypes["BUILDING_JAR_BLESSING2C"]
local blessing3C = GameInfoTypes["BUILDING_JAR_BLESSING3C"]
local blessing4C = GameInfoTypes["BUILDING_JAR_BLESSING4C"]
local blessing5C = GameInfoTypes["BUILDING_JAR_BLESSING5C"]
local blessing6C = GameInfoTypes["BUILDING_JAR_BLESSING6C"]
local blessing7C = GameInfoTypes["BUILDING_JAR_BLESSING7C"]
local tDumBlessingsC = {}
tDumBlessingsC[blessing1C] = true
tDumBlessingsC[blessing2C] = true
tDumBlessingsC[blessing3C] = true
tDumBlessingsC[blessing4C] = true
tDumBlessingsC[blessing5C] = true
tDumBlessingsC[blessing6C] = true
tDumBlessingsC[blessing7C] = true
--
local blessing1D = GameInfoTypes["BUILDING_JAR_BLESSING1D"]
local blessing2D = GameInfoTypes["BUILDING_JAR_BLESSING2D"]
local blessing3D = GameInfoTypes["BUILDING_JAR_BLESSING3D"]
local blessing4D = GameInfoTypes["BUILDING_JAR_BLESSING4D"]
local blessing5D = GameInfoTypes["BUILDING_JAR_BLESSING5D"]
local blessing6D = GameInfoTypes["BUILDING_JAR_BLESSING6D"]
local blessing7D = GameInfoTypes["BUILDING_JAR_BLESSING7D"]
local tDumBlessingsD = {}
tDumBlessingsD[blessing1D] = true
tDumBlessingsD[blessing2D] = true
tDumBlessingsD[blessing3D] = true
tDumBlessingsD[blessing4D] = true
tDumBlessingsD[blessing5D] = true
tDumBlessingsD[blessing6D] = true
tDumBlessingsD[blessing7D] = true
--
local blessing1E = GameInfoTypes["BUILDING_JAR_BLESSING1E"]
local blessing2E = GameInfoTypes["BUILDING_JAR_BLESSING2E"]
local blessing3E = GameInfoTypes["BUILDING_JAR_BLESSING3E"]
local blessing4E = GameInfoTypes["BUILDING_JAR_BLESSING4E"]
local blessing5E = GameInfoTypes["BUILDING_JAR_BLESSING5E"]
local blessing6E = GameInfoTypes["BUILDING_JAR_BLESSING6E"]
local blessing7E = GameInfoTypes["BUILDING_JAR_BLESSING7E"]
local tDumBlessingsE = {}
tDumBlessingsE[blessing1E] = true
tDumBlessingsE[blessing2E] = true
tDumBlessingsE[blessing3E] = true
tDumBlessingsE[blessing4E] = true
tDumBlessingsE[blessing5E] = true
tDumBlessingsE[blessing6E] = true
tDumBlessingsE[blessing7E] = true
---------------------------------
local tUnifiedBlessings = {}
tUnifiedBlessings[blessing1] = {}
tUnifiedBlessings[blessing2] = {}
tUnifiedBlessings[blessing3] = {}
tUnifiedBlessings[blessing4] = {}
tUnifiedBlessings[blessing5] = {}
tUnifiedBlessings[blessing6] = {}
tUnifiedBlessings[blessing7] = {}
	
tUnifiedBlessings[blessing1A] = {}
tUnifiedBlessings[blessing2A] = {}
tUnifiedBlessings[blessing3A] = {}
tUnifiedBlessings[blessing4A] = {}
tUnifiedBlessings[blessing5A] = {}
tUnifiedBlessings[blessing6A] = {}
tUnifiedBlessings[blessing7A] = {}
	
tUnifiedBlessings[blessing1B] = {}
tUnifiedBlessings[blessing2B] = {}
tUnifiedBlessings[blessing3B] = {}
tUnifiedBlessings[blessing4B] = {}
tUnifiedBlessings[blessing5B] = {}
tUnifiedBlessings[blessing6B] = {}
tUnifiedBlessings[blessing7B] = {}

tUnifiedBlessings[blessing1C] = {}
tUnifiedBlessings[blessing2C] = {}
tUnifiedBlessings[blessing3C] = {}
tUnifiedBlessings[blessing4C] = {}
tUnifiedBlessings[blessing5C] = {}
tUnifiedBlessings[blessing6C] = {}
tUnifiedBlessings[blessing7C] = {}

tUnifiedBlessings[blessing1D] = {}
tUnifiedBlessings[blessing2D] = {}
tUnifiedBlessings[blessing3D] = {}
tUnifiedBlessings[blessing4D] = {}
tUnifiedBlessings[blessing5D] = {}
tUnifiedBlessings[blessing6D] = {}
tUnifiedBlessings[blessing7D] = {}

tUnifiedBlessings[blessing1E] = {}
tUnifiedBlessings[blessing2E] = {}
tUnifiedBlessings[blessing3E] = {}
tUnifiedBlessings[blessing4E] = {}
tUnifiedBlessings[blessing5E] = {}
tUnifiedBlessings[blessing6E] = {}
tUnifiedBlessings[blessing7E] = {}
-----------------------------------

local tDumBlessings1 = {}
tDumBlessings1[blessing1] = true
tDumBlessings1[blessing1A] = true
tDumBlessings1[blessing1B] = true
tDumBlessings1[blessing1C] = true
tDumBlessings1[blessing1D] = true
tDumBlessings1[blessing1E] = true

local tDumBlessings2 = {}
tDumBlessings2[blessing2] = true
tDumBlessings2[blessing2A] = true
tDumBlessings2[blessing2B] = true
tDumBlessings2[blessing2C] = true
tDumBlessings2[blessing2D] = true
tDumBlessings2[blessing2E] = true

local tDumBlessings3 = {}
tDumBlessings3[blessing3] = true
tDumBlessings3[blessing3A] = true
tDumBlessings3[blessing3B] = true
tDumBlessings3[blessing3C] = true
tDumBlessings3[blessing3D] = true
tDumBlessings3[blessing3E] = true

local tDumBlessings4 = {}
tDumBlessings4[blessing4] = true
tDumBlessings4[blessing4A] = true
tDumBlessings4[blessing4B] = true
tDumBlessings4[blessing4C] = true
tDumBlessings4[blessing4D] = true
tDumBlessings4[blessing4E] = true

local tDumBlessings5 = {}
tDumBlessings5[blessing5] = true
tDumBlessings5[blessing5A] = true
tDumBlessings5[blessing5B] = true
tDumBlessings5[blessing5C] = true
tDumBlessings5[blessing5D] = true
tDumBlessings5[blessing5E] = true

local tDumBlessings6 = {}
tDumBlessings6[blessing6] = true
tDumBlessings6[blessing6A] = true
tDumBlessings6[blessing6B] = true
tDumBlessings6[blessing6C] = true
tDumBlessings6[blessing6D] = true
tDumBlessings6[blessing6E] = true

local tDumBlessings7 = {}
tDumBlessings7[blessing7] = true
tDumBlessings7[blessing7A] = true
tDumBlessings7[blessing7B] = true
tDumBlessings7[blessing7C] = true
tDumBlessings7[blessing7D] = true
tDumBlessings7[blessing7E] = true


local tDumFeature = {}
tDumFeature[GameInfoTypes.FEATURE_OASIS] = true
tDumFeature[GameInfoTypes.FEATURE_FLOOD_PLAINS] = true
tDumFeature[GameInfoTypes.FEATURE_FOREST] = true
tDumFeature[GameInfoTypes.FEATURE_CRATER] = true
tDumFeature[GameInfoTypes.FEATURE_FUJI] = true
tDumFeature[GameInfoTypes.FEATURE_MESA] = true
tDumFeature[GameInfoTypes.FEATURE_REEF] = true
tDumFeature[GameInfoTypes.FEATURE_VOLCANO] = true
tDumFeature[GameInfoTypes.FEATURE_GIBRALTAR] = true
tDumFeature[GameInfoTypes.FEATURE_GEYSER] = true
tDumFeature[GameInfoTypes.FEATURE_FOUNTAIN_YOUTH] = true
tDumFeature[GameInfoTypes.FEATURE_POTOSI] = true
tDumFeature[GameInfoTypes.FEATURE_EL_DORADO] = true
tDumFeature[GameInfoTypes.FEATURE_SRI_PADA] = true
tDumFeature[GameInfoTypes.FEATURE_MT_SINAI] = true
tDumFeature[GameInfoTypes.FEATURE_MT_KAILASH] = true
tDumFeature[GameInfoTypes.FEATURE_ULURU] = true
tDumFeature[GameInfoTypes.FEATURE_LAKE_VICTORIA] = true
tDumFeature[GameInfoTypes.FEATURE_KILIMANJARO] = true
tDumFeature[GameInfoTypes.FEATURE_SOLOMONS_MINES] = true
--------------------------------------------------------------------------------------------------------------------------
-- FUNCTIONS
--------------------------------------------------------------------------------------------------------------------------
-- HOZHOOJI UA
--========================================================================================================================
-- +1 Faith and +1 local happiness from great works of Music
function HappyFaithMusic(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		if (pPlayer:GetNumCities() > 1) then
			for pCity in pPlayer:Cities() do
				local GWMcount2 = 0
				for k, v in pairs(tBuildingGWMusic) do
					if pCity:IsHasBuilding(k) then
						GWMcount2= GWMcount2 + pCity:GetNumGreatWorksInBuilding(v)
					end
				end
				pCity:SetNumRealBuilding(iHappyFaithDummy,  GWMcount2)
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then GameEvents.PlayerDoTurn.Add(HappyFaithMusic) end;

-- Ceremonies with different production and faith cost unlocks in different eras
for row in GameInfo.Eras() do
	if row.Type == "ERA_CLASSICAL" then
		tUnifiedBlessings[blessing1A][row.ID] = true
		tUnifiedBlessings[blessing2A][row.ID] = true
		tUnifiedBlessings[blessing3A][row.ID] = true
		tUnifiedBlessings[blessing4A][row.ID] = true
		tUnifiedBlessings[blessing5A][row.ID] = true
		tUnifiedBlessings[blessing6A][row.ID] = true
		tUnifiedBlessings[blessing7A][row.ID] = true
	elseif row.ID >= GameInfoTypes["ERA_MODERN"] then
		tUnifiedBlessings[blessing1D][row.ID] = true
		tUnifiedBlessings[blessing2D][row.ID] = true
		tUnifiedBlessings[blessing3D][row.ID] = true
		tUnifiedBlessings[blessing4D][row.ID] = true
		tUnifiedBlessings[blessing5D][row.ID] = true
		tUnifiedBlessings[blessing6D][row.ID] = true
		tUnifiedBlessings[blessing7D][row.ID] = true
	elseif row.Type == "ERA_ANCIENT" then
		tUnifiedBlessings[blessing1E][row.ID] = true
		tUnifiedBlessings[blessing2E][row.ID] = true
		tUnifiedBlessings[blessing3E][row.ID] = true
		tUnifiedBlessings[blessing4E][row.ID] = true
		tUnifiedBlessings[blessing5E][row.ID] = true
		tUnifiedBlessings[blessing6E][row.ID] = true
		tUnifiedBlessings[blessing7E][row.ID] = true
	elseif row.Type == "ERA_MEDIEVAL" then
		tUnifiedBlessings[blessing1][row.ID] = true
		tUnifiedBlessings[blessing2][row.ID] = true
		tUnifiedBlessings[blessing3][row.ID] = true
		tUnifiedBlessings[blessing4][row.ID] = true
		tUnifiedBlessings[blessing5][row.ID] = true
		tUnifiedBlessings[blessing6][row.ID] = true
		tUnifiedBlessings[blessing7][row.ID] = true
	elseif row.Type == "ERA_RENAISSANCE" then
		tUnifiedBlessings[blessing1B][row.ID] = true
		tUnifiedBlessings[blessing2B][row.ID] = true
		tUnifiedBlessings[blessing3B][row.ID] = true
		tUnifiedBlessings[blessing4B][row.ID] = true
		tUnifiedBlessings[blessing5B][row.ID] = true
		tUnifiedBlessings[blessing6B][row.ID] = true
		tUnifiedBlessings[blessing7B][row.ID] = true
	elseif row.Type == "ERA_INDUSTRIAL" then
		tUnifiedBlessings[blessing1C][row.ID] = true
		tUnifiedBlessings[blessing2C][row.ID] = true
		tUnifiedBlessings[blessing3C][row.ID] = true
		tUnifiedBlessings[blessing4C][row.ID] = true
		tUnifiedBlessings[blessing5C][row.ID] = true
		tUnifiedBlessings[blessing6C][row.ID] = true
		tUnifiedBlessings[blessing7C][row.ID] = true
	elseif row.Type == "ERA_ENLIGHTENMENT" then
		tUnifiedBlessings[blessing1C][row.ID] = true
		tUnifiedBlessings[blessing2C][row.ID] = true
		tUnifiedBlessings[blessing3C][row.ID] = true
		tUnifiedBlessings[blessing4C][row.ID] = true
		tUnifiedBlessings[blessing5C][row.ID] = true
		tUnifiedBlessings[blessing6C][row.ID] = true
		tUnifiedBlessings[blessing7C][row.ID] = true
	end
end

function C15_NavajoBlessingCeremonies(playerID, buildingID)
	local pPlayer = Players[playerID]
	if pPlayer:GetCivilizationType() ~= iCiv and tUnifiedBlessings[buildingID] then return false end
	if pPlayer:GetCivilizationType() == iCiv and tUnifiedBlessings[buildingID] then
		if tUnifiedBlessings[buildingID][pPlayer:GetCurrentEra()] then
			return true
		else
			return false
		end
	end
	return true
end

GameEvents.PlayerCanConstruct.Add(C15_NavajoBlessingCeremonies)



function SetChickenTimer(iPlayer, iCity, eBuilding )
	local pPlayer = Players[iPlayer]
	local pCity = pPlayer:GetCityByID(iCity)
	local pPlot = pCity:Plot()
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		local factor = 0
		if pPlayer:GetCurrentEra() == GameInfoTypes["ERA_ANCIENT"] then
			factor = 5
		elseif pPlayer:GetCurrentEra() == GameInfoTypes["ERA_CLASSICAL"] then
			factor = 4
		elseif pPlayer:GetCurrentEra() == GameInfoTypes["ERA_MEDIEVAL"] then
			factor = 3
		elseif pPlayer:GetCurrentEra() > GameInfo.Eras["ERA_MEDIEVAL"].ID and pPlayer:GetCurrentEra() < GameInfo.Eras["ERA_INDUSTRIAL"].ID then
			factor = 2
		elseif pPlayer:GetCurrentEra() == GameInfoTypes["ERA_INDUSTRIAL"] then
			factor = 1
		elseif pPlayer:GetCurrentEra() > GameInfo.Eras["ERA_INDUSTRIAL"].ID then
			factor = 0
		end
	
		local standard = 5 + factor + GetNumGreatWorkMusic(pPlayer) + prophetcount
		local turns = standard*iMod
		local cap = (25 + prophetcount)*iMod
		if turns>cap  then turns=math.floor(cap) end
		local citypop = pCity:GetPopulation()
		local GMusicianPoints = turns *citypop *10
		if tDumBlessings1[eBuilding] then 
		--if eBuilding == blessing1 or eBuilding == blessing1A or eBuilding == blessing1B or eBuilding == blessing1C or eBuilding == blessing1D or eBuilding == blessing1E then --Gameway 
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
		
			
			local countA = 0 
			for adjacentPlot in PlotAreaSweepIterator(pPlot, 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
				if adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_COW or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_BISON or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_SHEEP or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_DEER then 
					countA = countA +1					
				end
			end
			if countA>=20 then
				pCity:SetNumRealBuilding(blessfood, 20)
			else
				pCity:SetNumRealBuilding(blessfood, countA)
			end
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "+" .. countA .. "% Pop Growth", 3)
		
		
		elseif tDumBlessings2[eBuilding] then 
		--elseif eBuilding == blessing2 or eBuilding == blessing2A or eBuilding == blessing2B or eBuilding == blessing2C or eBuilding == blessing2D or eBuilding == blessing2E then -- Lifeway
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
		
			local countH = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if zPlot:GetImprovementType() == hoganID then
					countH = countH +1
				end
			end
			if countH>=5 then
				pCity:SetNumRealBuilding(blessprod, 5)
			else
				pCity:SetNumRealBuilding(blessprod, countH)
			end
			
			local countW = 0
			if pPlot:IsUnit() then
				for i = 0, (pPlot:GetNumUnits() - 1) do
					local pUnit = pPlot:GetUnit(i)
					if pUnit then
						local workrate = pUnit:WorkRate()
						if pUnit:GetOwner() == iPlayer and workrate >1 then
							countW = countW +1
						end
					end
				end
			end					
			pCity:SetNumRealBuilding(blessprod2, 2 + countW)
		
		
		elseif tDumBlessings3[eBuilding] then 
		--elseif eBuilding == blessing3 or eBuilding == blessing3A or eBuilding == blessing3B or eBuilding == blessing3C or eBuilding == blessing3D or eBuilding == blessing3E then --Beautyway
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
			pPlayer:ChangeGoldenAgeProgressMeter(math.ceil(citypop*0.25))
		
			local countB = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if	tDumFeature[ zPlot:GetFeatureType() ] then 
					countB = countB +1
				end						
				if zPlot:IsMountain() or zPlot:GetPlotType() == PlotTypes.PLOT_MOUNTAIN then
					countB = countB +1
				end
			end
			pCity:SetNumRealBuilding(blessbeautydum, math.ceil(countB*0.25))
			if not isUsingCBP then
				Jarcast_SetBinaryTourismBuildings( pCity, math.ceil(countB*0.25))
			end
		
		elseif tDumBlessings4[eBuilding] then 
		--elseif eBuilding == blessing4 or eBuilding == blessing4A or eBuilding == blessing4B or eBuilding == blessing4C or eBuilding == blessing4D or eBuilding == blessing4E then --Enemyway
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
		
		
			if not isUsingCBP then
				pCity:SetNumRealBuilding(blessfaith, math.floor(citypop*0.2))
			end
		
		elseif tDumBlessings5[eBuilding] then 
		--elseif eBuilding == blessing5 or eBuilding == blessing5A or eBuilding == blessing5B or eBuilding == blessing5C or eBuilding == blessing5D or eBuilding == blessing5E then -- Protectionway
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
			
			pCity:SetNumRealBuilding(iEnemywayDummy,  math.ceil(citypop*0.25))
		
		elseif tDumBlessings6[eBuilding] then 
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
			
			
			local countH = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if zPlot:GetImprovementType() == hoganID then
					countH = countH +1
				end
			end
			if countH>=5 then
				pCity:SetNumRealBuilding(blessscimod, 5)
			else
				pCity:SetNumRealBuilding(blessscimod, countH)
			end
					
			local count6 = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if	(zPlot:GetFeatureType() == -1) then
					if zPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT or  zPlot:GetTerrainType()== TerrainTypes.TERRAIN_PLAINS then
						count6 = count6 +1
					end
				end
			end
			pCity:SetNumRealBuilding(blesssci2, math.ceil(count6*0.5))
			
		elseif tDumBlessings7[eBuilding] then 
			pCity:SetNumRealBuilding(iTimerDummy,  turns +1)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], GMusicianPoints)
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. GMusicianPoints/100 .. "[ENDCOLOR][ICON_GREAT_MUSICIAN]", 2)
			
			local countH = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if zPlot:GetImprovementType() == hoganID then
					countH = countH +1
				end
			end
			if countH>=5 then
				pCity:SetNumRealBuilding(blessgoldmod, 5)
			else
				pCity:SetNumRealBuilding(blessgoldmod, countH)
			end
			
			local count7 = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				-- rest of code
				if	(zPlot:GetFeatureType() == -1) then
					if zPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT or  zPlot:GetTerrainType()== TerrainTypes.TERRAIN_PLAINS then
						count7 = count7 +1
					end
				end
			end
			pCity:SetNumRealBuilding(blessgold2, math.ceil(count7*0.5))
			
		else
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then GameEvents.CityConstructed.Add(SetChickenTimer) end;


function TicTokChickenClock(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		for pCity in pPlayer:Cities() do
			if pCity:IsHasBuilding(iTimerDummy) then
				local num= pCity:GetNumBuilding(iTimerDummy)
				if num >1 then
					pCity:SetNumRealBuilding(iTimerDummy,  num -1 )
					if pPlayer:IsHuman() then
						Events.GameplayAlertMessage('Blessing in [COLOR_WARNING_TEXT]' .. pCity:GetName() .. '[ENDCOLOR]: [COLOR_WARNING_TEXT]'.. num -1 .. "[ENDCOLOR] turns to end")
					end
				else
					pCity:SetNumRealBuilding(iTimerDummy,  0)
					if pPlayer:IsHuman() then
						Events.GameplayAlertMessage('Blessing ceremony ended in [COLOR_WARNING_TEXT]' .. pCity:GetName() .. '[ENDCOLOR].')
					end
					
					for k, v in pairs(tDumBlessings) do
						pCity:SetNumRealBuilding(k, 0)
					end
					for k, v in pairs(tDumBlessingsA) do
						pCity:SetNumRealBuilding(k, 0)
					end
					for k, v in pairs(tDumBlessingsB) do
						pCity:SetNumRealBuilding(k, 0)
					end
					for k, v in pairs(tDumBlessingsC) do
						pCity:SetNumRealBuilding(k, 0)
					end
					for k, v in pairs(tDumBlessingsD) do
						pCity:SetNumRealBuilding(k, 0)
					end					
					for k, v in pairs(tDumBlessingsE) do
						pCity:SetNumRealBuilding(k, 0)
					end
					
					pCity:SetNumRealBuilding(blessfood, 0)
					pCity:SetNumRealBuilding(blessprod, 0)
					pCity:SetNumRealBuilding(blessprod2, 0)
					pCity:SetNumRealBuilding(blessbeautydum, 0)
					pCity:SetNumRealBuilding(blessfaith, 0)
					pCity:SetNumRealBuilding(blessscimod, 0)
					pCity:SetNumRealBuilding(blesssci2, 0)
					pCity:SetNumRealBuilding(blessgoldmod, 0)
					pCity:SetNumRealBuilding(blessgold2, 0)
					pCity:SetNumRealBuilding(iEnemywayDummy, 0)
					Jarcast_SetBinaryTourismBuildings( pCity, 0)
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then GameEvents.PlayerDoTurn.Add(TicTokChickenClock) end;


function BlessingBonuses(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		for pCity in pPlayer:Cities() do
		
			for k, v in pairs(tDumBlessings1) do
				if pCity:IsHasBuilding(k) then
					local pPlot = pCity:Plot()
					local countA = 0 
					for adjacentPlot in PlotAreaSweepIterator(pPlot, 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
						if adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_COW or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_BISON or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_SHEEP or adjacentPlot:GetResourceType() == GameInfoTypes.RESOURCE_DEER then 
							countA = countA +1					
						end
					end
					if countA>=20 then
						pCity:SetNumRealBuilding(blessfood, 20)
					else
						pCity:SetNumRealBuilding(blessfood, countA)
					end
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()))), "+" .. countA .. "% Pop Growth", 3)
				end
			end
			
			for k, v in pairs(tDumBlessings2) do
				if pCity:IsHasBuilding(k) then
					local countH = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if zPlot:GetImprovementType() == hoganID then
							countH = countH +1
						end
					end
					if countH>=5 then
						pCity:SetNumRealBuilding(blessprod, 5)
					else
						pCity:SetNumRealBuilding(blessprod, countH)
					end
					
					local pPlot = pCity:Plot()
					local countW = 0
					if pPlot:IsUnit() then
						for i = 0, (pPlot:GetNumUnits() - 1) do
							local pUnit = pPlot:GetUnit(i)
							if pUnit then
								local workrate = pUnit:WorkRate()
								if pUnit:GetOwner() == iPlayer and workrate >1 then
									countW = countW +1
								end
							end
						end
					end					
					pCity:SetNumRealBuilding(blessprod2, 2 + countW)
				end
			end
		
			for k, v in pairs(tDumBlessings3) do
				if pCity:IsHasBuilding(k) then
					local countB = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if	tDumFeature[ zPlot:GetFeatureType() ] then 
							countB = countB +1
						end						
						if zPlot:IsMountain() or zPlot:GetPlotType() == PlotTypes.PLOT_MOUNTAIN then
							countB = countB +1
						end
					end
					pCity:SetNumRealBuilding(blessbeautydum, math.ceil(countB*0.25))
					if not isUsingCBP then
						Jarcast_SetBinaryTourismBuildings( pCity, math.ceil(countB*0.25))
					end
				end
			end
			
		
			for k, v in pairs(tDumBlessings4) do
				if pCity:IsHasBuilding(k) then
					local FaithYield = pCity:GetYieldRate(YieldTypes.YIELD_FAITH)
					if pCity:IsProductionUnit() then
						local unitProduction = pCity:GetProductionUnit()
						pCity:ChangeUnitProduction(unitProduction, FaithYield)
						--Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "[COLOR_POPUP_TEXT][ICON_FOOD] Food added to [ICON_PRODUCTION] Unit Production[ENDCOLOR]", 3)
					end
					if not isUsingCBP then
						pCity:SetNumRealBuilding(blessfaith, math.floor(pCity:GetPopulation() *0.2))
					end
				end
			end
		
			for k, v in pairs(tDumBlessings5) do
				if pCity:IsHasBuilding(k) then
					pCity:SetNumRealBuilding(iEnemywayDummy,  math.ceil(pCity:GetPopulation() *0.25))
				end
			end
		
			for k, v in pairs(tDumBlessings6) do
				if pCity:IsHasBuilding(k) then
					local countH = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if zPlot:GetImprovementType() == hoganID then
							countH = countH +1
						end
					end
					if countH>=5 then
						pCity:SetNumRealBuilding(blessscimod, 5)
					else
						pCity:SetNumRealBuilding(blessscimod, countH)
					end
					
					local count6 = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if	(zPlot:GetFeatureType() == -1) then
							if zPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT or  zPlot:GetTerrainType()== TerrainTypes.TERRAIN_PLAINS then
								count6 = count6 +1
							end
						end
					end
					pCity:SetNumRealBuilding(blesssci2, math.ceil(count6*0.5))
				end
			end
		
			for k, v in pairs(tDumBlessings7) do
				if pCity:IsHasBuilding(k) then
					local countH = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if zPlot:GetImprovementType() == hoganID then
							countH = countH +1
						end
					end
					if countH>=5 then
						pCity:SetNumRealBuilding(blessgoldmod, 5)
					else
						pCity:SetNumRealBuilding(blessgoldmod, countH)
					end
					
					local count7 = 0
					for i = 0, pCity:GetNumCityPlots() - 1, 1 do
						local zPlot = pCity:GetCityIndexPlot(i)
						-- rest of code
						if	(zPlot:GetFeatureType() == -1) then
							if zPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT or  zPlot:GetTerrainType()== TerrainTypes.TERRAIN_PLAINS then
								count7 = count7 +1
							end
						end
					end
					pCity:SetNumRealBuilding(blessgold2, math.ceil(count7*0.5))
				end
			end
		end
	end
end
		
if JFD_IsCivilisationActive(iCiv) then GameEvents.PlayerDoTurn.Add(BlessingBonuses) end;


function GetResourceClassFromID(identifier)
	for row in GameInfo.Resources("ID = '" .. identifier .. "'") do
		return row.ResourceClassType
	end
end

function NavajoBorderExpansion(pPlayer, pCity)
	local iTeam = pPlayer:GetTeam()
	-- creation of matrix to fill with eligible plots
	local tEligiblePlots = {}
	for i=1,5 do -- Plots ranked by 5 levels of priority depending on resources: strategic, then luxuries, then bonus, then resourceless with and without feature
		tEligiblePlots[i] = {}
	end
	local count1 = 0
	local count2 = 0
	local count3 = 0
	local count4 = 0
	local count5 = 0
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		for k, v in pairs(tDumBlessings4) do
			if pCity:IsHasBuilding(k) then
			--if pCity:IsHasBuilding(blessing4) or pCity:IsHasBuilding(blessing4A) or pCity:IsHasBuilding(blessing4B) or pCity:IsHasBuilding(blessing4C) or pCity:IsHasBuilding(blessing4D) or pCity:IsHasBuilding(blessing4E) then
				for zPlot in PlotAreaSweepIterator(pPlot, 5, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
					if zPlot:GetOwner() == iPlayer then
						print("Navajo Nightway: |=    | First Loop: Owned plots")
						for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
							print("Navajo Nightway: |==   | Second loop: Adjacent plots")
							local pAdjacentPlot = Map.PlotDirection(zPlot:GetX(), zPlot:GetY(), direction)
							if pAdjacentPlot and pAdjacentPlot:GetOwner() < 0 then
								local xAdjPlot = pAdjacentPlot:GetX()
								local yAdjPlot = pAdjacentPlot:GetY()
								local plotResource = pAdjacentPlot:GetResourceType()
								local distance = Map.PlotDistance(xAdjPlot, yAdjPlot, xcity, ycity)
								print("Navajo Nightway: |===  | Neutral tile constraint.", xAdjPlot, yAdjPlot, " Distance: ", distance)
								if distance < 6 then
									print("Navajo Nightway: |==== | Distance constraint")
									if plotResource == -1 then
										if pAdjacentPlot:GetFeatureType() == -1 then
											count1 = count1 + 1
											tEligiblePlots[1][count1] = {}
											tEligiblePlots[1][count1].x = xAdjPlot
											tEligiblePlots[1][count1].y = yAdjPlot
											print("Navajo Nightway: |=====| Resourceless w/o feature plot eligible.. ", xAdjPlot, yAdjPlot)
										else
											count2 = count2 + 1
											tEligiblePlots[2][count2] = {}
											tEligiblePlots[2][count2].x = xAdjPlot
											tEligiblePlots[2][count2].y = yAdjPlot
											print("Navajo Nightway: |=====| Resourceless w/ feature plot eligible.. ", xAdjPlot, yAdjPlot)
										end
											
									elseif 	GetResourceClassFromID( plotResource ) == "RESOURCECLASS_BONUS" then
										count3 = count3 + 1
										tEligiblePlots[3][count3] = {}
										tEligiblePlots[3][count3].x = xAdjPlot
										tEligiblePlots[3][count3].y = yAdjPlot
										print("Navajo Nightway: |=====| BonusRes plot eligible.. ", xAdjPlot, yAdjPlot)
									elseif  GetResourceClassFromID( plotResource ) == "RESOURCECLASS_LUXURY" then
										count4 = count4 + 1
										tEligiblePlots[4][count4] = {}
										tEligiblePlots[4][count4].x = xAdjPlot
										tEligiblePlots[4][count4].y = yAdjPlot
										print("Navajo Nightway: |=====| LuxuryRes plot eligible.. ", xAdjPlot, yAdjPlot)
									else
										count5 = count5 + 1
										tEligiblePlots[5][count5] = {}
										tEligiblePlots[5][count5].x = xAdjPlot
										tEligiblePlots[5][count5].y = yAdjPlot
										print("Navajo Nightway: |=====| StratRes plot eligible.. ", xAdjPlot, yAdjPlot)
									end
								end
							end
						end	
					end
				end
				-- Strategic resource plots
				if (count5 > 0) then	
					local rand = GetRandom(1, count5);
					local ElectedPlotX = tEligiblePlots[5][rand].x 
					local ElectedPlotY = tEligiblePlots[5][rand].y
					local pElectedPlot = Map.GetPlot(ElectedPlotX, ElectedPlotY)
					pElectedPlot:SetOwner(iPlayer, iCity, true, true);
					pElectedPlot:SetRevealed(iTeam, true)
					-- Notification
					local sTitle = "Borders expand in " .. pCity:GetName() .. " during Nightway."
					local sText = "While celebrating the Nightway ritual, " .. pCity:GetName() .. "acquired new lands."
					pPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, ElectedPlotX, ElectedPlotY)
				-- Luxury resource plots
				elseif count5 == 0 and (count4 > 0) then
					local rand = GetRandom(1, count4);
					local ElectedPlotX = tEligiblePlots[4][rand].x 
					local ElectedPlotY = tEligiblePlots[4][rand].y
					local pElectedPlot = Map.GetPlot(ElectedPlotX, ElectedPlotY)
					pElectedPlot:SetOwner(iPlayer, iCity, true, true);
					pElectedPlot:SetRevealed(iTeam, true)
					-- Notification
					local sTitle = "Borders expand in " .. pCity:GetName() .. " during Nightway."
					local sText = "While celebrating the Nightway ritual, " .. pCity:GetName() .. "acquired new lands."
					pPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, ElectedPlotX, ElectedPlotY)
				-- Bonus resource plots
				elseif count5 == 0 and count4 == 0 and count3 > 0 then
					local rand = GetRandom(1, count3);
					local ElectedPlotX = tEligiblePlots[3][rand].x 
					local ElectedPlotY = tEligiblePlots[3][rand].y
					local pElectedPlot = Map.GetPlot(ElectedPlotX, ElectedPlotY)
					pElectedPlot:SetOwner(iPlayer, iCity, true, true);
					pElectedPlot:SetRevealed(iTeam, true)
					-- Notification
					local sTitle = "Borders expand in " .. pCity:GetName() .. " during Nightway."
					local sText = "While celebrating the Nightway ritual, " .. pCity:GetName() .. "acquired new lands."
					pPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, ElectedPlotX, ElectedPlotY)
				-- Resourceless plots with features
				elseif count5 == 0 and count4 == 0 and count3 == 0 and count2 > 0 then
					local rand = GetRandom(1, count2);
					local ElectedPlotX = tEligiblePlots[2][rand].x 
					local ElectedPlotY = tEligiblePlots[2][rand].y
					local pElectedPlot = Map.GetPlot(ElectedPlotX, ElectedPlotY)
					pElectedPlot:SetOwner(iPlayer, iCity, true, true);
					pElectedPlot:SetRevealed(iTeam, true)
					-- Notification
					local sTitle = "Borders expand in " .. pCity:GetName() .. " during Nightway."
					local sText = "While celebrating the Nightway ritual, " .. pCity:GetName() .. "acquired new lands."
					pPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, ElectedPlotX, ElectedPlotY)
				-- Only resourceless plots w/o features
				elseif count5 == 0 and count4 == 0 and count3 == 0 and count2 == 0 and count1 > 0 then
					local rand = GetRandom(1, count1);
					local ElectedPlotX = tEligiblePlots[1][rand].x 
					local ElectedPlotY = tEligiblePlots[1][rand].y
					local pElectedPlot = Map.GetPlot(ElectedPlotX, ElectedPlotY)
					pElectedPlot:SetOwner(iPlayer, iCity, true, true);
					pElectedPlot:SetRevealed(iTeam, true)
					-- Notification
					local sTitle = "Borders expand in " .. pCity:GetName() .. " during Nightway."
					local sText = "While celebrating the Nightway ritual, " .. pCity:GetName() .. "acquired new lands."
					pPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, ElectedPlotX, ElectedPlotY)
				else
				end
				print("Navajo Nightway: Selected plot is: ", ElectedPlotX, ElectedPlotY)
			end
		end
	end
end

function Bless4OnCityPopulationChanged(x, y, newPop)
	local pPlot = Map.GetPlot(ToGridFromHex(x, y))
	local pCity = pPlot:GetPlotCity()
	local pPlayer = Players[pCity:GetOwner()]
	NavajoBorderExpansion(pPlayer, pCity)
end

function Bless4OnUnitTrainedInCity(ownerId, cityId, unitId, bGold, bFaithOrCulture)
	local pPlayer = Players[ownerId]
	local pCity = pPlayer:GetCityByID(cityId)
	local pUnit = pPlayer:GetUnitByID(unitId)
	if pUnit:GetBaseCombatStrength() == 0 and pUnit:GetBaseRangedCombatStrength()== 0 then
		NavajoBorderExpansion(pPlayer, pCity)
	end
end
		
if JFD_IsCivilisationActive(iCiv) then
	Events.SerialEventCityPopulationChanged.Add(Bless4OnCityPopulationChanged)
	GameEvents.CityTrained.Add(Bless4OnUnitTrainedInCity);
end;
--==========================================================================================================================
-- HATAALII functions

-- Trigger Holyway blessing when expended
function HataaliiExpended(iPlayer, eGreatPersonType)
	local pPlayer = Players[iPlayer]
	local capital = pPlayer:GetCapitalCity()
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		local standard = 10 + GetNumGreatWorkMusic(pPlayer) + prophetcount
		local turns = standard*iMod
		local cap = (30+ prophetcount)*iMod
		if turns>cap  then turns= math.floor(cap) end
		local pop= pPlayer:GetTotalPopulation()
		local bonus= math.ceil(pop*turns*0.20)
		if eGreatPersonType== GameInfoTypes.UNIT_JAR_HATAALII then 
			pPlayer:ChangeGoldenAgeProgressMeter(bonus)
			--[[pPlayer:ChangeCombatExperience(bonus) -- Great General
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MUSICIAN"], bonus*100)
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_ARTIST"], bonus*100)
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_WRITER"], bonus*100)
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MERCHANT"], bonus*100)
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_ENGINEER"], bonus*100)
			capital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_SCIENTIST"], bonus*100)]]
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(capital:Plot():GetX(), capital:Plot():GetY()))), "Holyway", 1)
			--Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(capital:Plot():GetX(), capital:Plot():GetY()))), "[COLOR_GREAT_PEOPLE_STORED]+" .. bonus .. "[ENDCOLOR][ICON_GREAT_PEOPLE]! [COLOR_GREAT_PEOPLE_STORED]+" .. bonus .. "[ENDCOLOR][ICON_GOLDEN_AGE]!", 2)
			-- medicine man at work when most needed
			--if C15_IsPlayerAtWarWithAnyone(iPlayer, false) and isUsingCBO then
				local heal = 33
				for pUnit in pPlayer:Units() do
					pUnit:ChangeDamage(-heal)
					--pUnit:SetDamage(0) -- completely heal
				end
			--end
			-- number of prophets increases the duration of blessings and its maximum
			if pPlayer:GetReligionCreatedByPlayer() ~= -1 then
				prophetcount = prophetcount +1
			end
		end
	end
end
	
if JFD_IsCivilisationActive(iCiv) then GameEvents.GreatPersonExpended.Add(HataaliiExpended) end;

-- Holy sites heal +15 HP per turn, +5 HP if adjacent
function NavajoHolySiteHealUnits(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then	
		for pCity in pPlayer:Cities() do
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pPlot = pCity:GetCityIndexPlot(i)
				if pPlot:GetImprovementType() == GameInfo.Improvements["IMPROVEMENT_HOLY_SITE"].ID and pPlot:GetOwner() == iPlayer then
					if pPlot:IsUnit() then
						for iPlotUnit = 0, pPlot:GetNumUnits() -1, 1 do
							local pUnit = pPlot:GetUnit(iPlotUnit)
							local iUnitOwner = pUnit:GetOwner()
							if iUnitOwner == iPlayer then
								pUnit:ChangeDamage(-15)
							end
						end
					end
					for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
						local pAdjacentPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), direction)
						if pAdjacentPlot and pAdjacentPlot:IsUnit() then
							for iPlotUnit = 0, pAdjacentPlot:GetNumUnits() -1, 1 do
								local pAdjUnit = pAdjacentPlot:GetUnit(iPlotUnit)
								local iAdjUnitOwner = pAdjUnit:GetOwner()
								if iAdjUnitOwner == iPlayer then
									pAdjUnit:ChangeDamage(-5)
								end
							end
						end
					end
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then
GameEvents.PlayerDoTurn.Add(NavajoHolySiteHealUnits)
end;
--==========================================================================================================================
-- HOGAN functions

-- Give a slot for great work of music to working city
function HoganMusicSlot(iPlayer)
	local pPlayer = Players[iPlayer]
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		for pCity in pPlayer:Cities() do
			local pPlot = pCity:Plot()
			local hogancount = 0
			for adjacentPlot in PlotAreaSweepIterator(pPlot, 3, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_INCLUDE) do
				x = adjacentPlot:GetX()
				y = adjacentPlot:GetY()
				if adjacentPlot:GetImprovementType() == hoganID then
					hogancount = hogancount +1
				end
			end
			if hogancount==1 then
				pCity:SetNumRealBuilding(hoganBuilding1,  1)
			elseif hogancount==2 then
				pCity:SetNumRealBuilding(hoganBuilding2,  1)	
			elseif hogancount==3 then
				pCity:SetNumRealBuilding(hoganBuilding3,  1)	
			elseif hogancount==4 then
				pCity:SetNumRealBuilding(hoganBuilding4,  1)	
			elseif hogancount==0 then
				pCity:SetNumRealBuilding(hoganBuilding1,  0)
				pCity:SetNumRealBuilding(hoganBuilding2,  0)
				pCity:SetNumRealBuilding(hoganBuilding3,  0)
				pCity:SetNumRealBuilding(hoganBuilding4,  0)
			else
			end
		end
	end
end
		
if JFD_IsCivilisationActive(iCiv) then
GameEvents.PlayerDoTurn.Add(HoganMusicSlot)
end;


-- Hogan gives +2 Food to own desert tiles
JarHoganDesertFood_BuildFinished_PlayerID = 0
JarHoganDesertFood_BuildFinished_X = 0
JarHoganDesertFood_BuildFinished_Y = 0
JarHoganDesertFood_BuildFinished_Improvement = 0
	
function HoganDesertFood(playerID, plotX, plotY, improvementID)
	if(JarHoganDesertFood_BuildFinished_PlayerID == playerID and JarHoganDesertFood_BuildFinished_X == plotX and JarHoganDesertFood_BuildFinished_Y == plotY and JarHoganDesertFood_BuildFinished_Improvement == improvementID) then
		return;
	end
	-- Store inputs for duplication check
	JarHoganDesertFood_BuildFinished_PlayerID = playerID;	
	JarHoganDesertFood_BuildFinished_X = plotX;					
	JarHoganDesertFood_BuildFinished_Y = plotY;					
	JarHoganDesertFood_BuildFinished_Improvement = improvementID;
	
	local pPlot = Map.GetPlot(plotX, plotY)
	local pPlayer = Players[playerID]
	--local pPlayer = Players[pPlot:GetOwner()]
	if pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive() then
		if improvementID == hoganID then
			if pPlot:GetTerrainType() == TerrainTypes.TERRAIN_DESERT then
				Game.SetPlotExtraYield(pPlot:GetX(), pPlot:GetY(), iFood, 2 )
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then
GameEvents.BuildFinished.Add(HoganDesertFood)
end
--==========================================================================================================================
-- CODE TALKER functions

function CodeTalkerPromotionActive(iPlayer, iUnitID)
	local pPlayer = Players[iPlayer]
	if NavajoHasTech(iTechreq) then
		if (pPlayer:GetCivilizationType() ~= iCiv and pPlayer:IsAlive()) then
			local pUnit = pPlayer:GetUnitByID(iUnitID)
			local unitplot = pUnit:GetPlot()
			local pOwner = Players[pUnit:GetOwner()];
			for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
				local pAdjacentPlot = Map.PlotDirection(unitplot:GetX(), unitplot:GetY(), direction)
				if pAdjacentPlot and pAdjacentPlot:IsUnit() then
					for iPlotUnit = 0, pAdjacentPlot:GetNumUnits() -1, 1 do
						local pAdjUnit = pAdjacentPlot:GetUnit(iPlotUnit)
						local iAdjUnitOwner = pAdjUnit:GetOwner()
						local pAdjUnitOwner = Players[iAdjUnitOwner]
						if not pAdjUnitOwner:IsBarbarian() then
							if pAdjUnit:IsHasPromotion(CodeTalkerPromoID) and pAdjUnitOwner:GetCivilizationType() ~= iCiv then
								if 		iAdjUnitOwner == iPlayer then
									pUnit:SetHasPromotion(CTBonusPromoID, true)
								elseif 	iAdjUnitOwner ~= iPlayer and PlayersShareWarEnemy(iAdjUnitOwner, iPlayer) then
									pUnit:SetHasPromotion(CTBonusPromoID, true)
								else
									pUnit:SetHasPromotion(CTMalusPromoID, true)
								end
							else
								pUnit:SetHasPromotion(CTBonusPromoID, false)
								pUnit:SetHasPromotion(CTMalusPromoID, false)
							end
						end
					end
				end
			end
		end
	end
end

if JFD_IsCivilisationActive(iCiv) then 
	GameEvents.UnitSetXY.Add(CodeTalkerPromotionActive)
end

-- When gifted away to City State CT produces influence
function CodeTalkerCSInfluence(iOwner, iUnitID, iUnitType, iX, iY, bDelay, iKiller)
	local pOwner = Players[iOwner]
	local pUnit = pOwner:GetUnitByID(iUnitID)
	local unitplot = pUnit:GetPlot()
	local influencegain = 5 + math.floor( pUnit:GetExperience()*0.33 )
	if unitplot:GetOwner() == -1 then return false end
	local pMinorCiv = Players[unitplot:GetOwner()]
	if (pOwner:GetCivilizationType() == iCiv and pOwner:IsAlive()) then
		if (iUnitType == CodeTalkerID ) and pUnit:GetCurrHitPoints() > 0 then
			if pMinorCiv:IsMinorCiv() then
				pMinorCiv:ChangeMinorCivFriendshipWithMajor(pOwner, influencegain)
			end
		end
	end
end
		
if JFD_IsCivilisationActive(iCiv) then
GameEvents.UnitPrekill.Add(CodeTalkerCSInfluence)
end;

-- Grant diplomatic bonus from the civ you gift Code Talker to
local CTDiploModifier = GameInfoTypes.DIPLOMODIFIER_JAR_NAVAJO_CODE_TALKER

function CodeTalkerDiploModifier(iDiploModifier, iFromPlayer, iToPlayer)
	local iModifier = 0
	local pToPlayer = Players[iToPlayer]
	local pFromPlayer = Players[iFromPlayer];
	if iDiploModifier == CTDiploModifier  and (Teams[pToPlayer:GetTeam()]:IsHasTech( iTechreq )) then
		print(string.format("CodeTalkerDiploModifier(%s, %i, %s, %s)", GameInfo.DiploModifiers[iDiploModifier].Type, iModifier, Players[iFromPlayer]:GetCivilizationDescription(), Players[iToPlayer]:GetCivilizationDescription()))
		if pFromPlayer:GetCivilizationType() ~= iCiv then
			local CTcount = 0
			for pUnit in pFromPlayer:Units() do
				if pUnit:IsHasPromotion(CodeTalkerPromoID) then
					CTcount = CTcount +1
				end
			end
			if CTcount <=10 then
				iModifier = -5 -1*CTcount;
			else
				iModifier = -15
			end
		end
	end
	return iModifier
end
	
if Game_IsUsingMod(CBPmodID) then
	GameEvents.GetDiploModifier.Add(CodeTalkerDiploModifier)
end
--==========================================================================================================================
-- NAABAAHII functions

local NaabaahiiPromoID = GameInfoTypes["PROMOTION_NAABAAHII"]

-- Naabaahii collects yields from tiles he pillages
function OnPillageCaptureTileYield(iPlayer, iUnit, iImprovement, iGold)
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		if pUnit and pUnit:IsHasPromotion(NaabaahiiPromoID) then
			local unitplot = pUnit:GetPlot()
			local yfood = unitplot:GetYield(YieldTypes.YIELD_FOOD)
			local yprod = unitplot:GetYield(YieldTypes.YIELD_PRODUCTION)
			local ygold = unitplot:GetYield(YieldTypes.YIELD_GOLD)
			local yscience = unitplot:GetYield(YieldTypes.YIELD_SCIENCE)
			local yculture = unitplot:GetYield(YieldTypes.YIELD_CULTURE)
			local yfaith = unitplot:GetYield(YieldTypes.YIELD_FAITH)
			if yculture>0 then
				pPlayer:ChangeJONSCulture(yculture)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "+" .. yculture .. " [ICON_CULTURE]", 1)
			end
			if ygold>0 then
				pPlayer:ChangeGold(ygold)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "+" .. ygold .. " [ICON_GOLD]", 2)
			end
			if yfaith>0 then
				pPlayer:ChangeFaith(yfaith)
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "+" .. yfaith .. " [ICON_PEACE]", 3)
			end
			if yscience>0 then
				local iTech = pPlayer:GetCurrentResearch()
				Teams[pPlayer:GetTeam()]:GetTeamTechs():ChangeResearchProgress(iTech, yscience, pPlayer:GetID())
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "+" .. yscience .. " [ICON_RESEARCH]", 4)
			end
				
			for city in pPlayer:Cities() do
				if yfood>0 then
					city:ChangeFood(yfood)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "+" .. yfood .. " [ICON_FOOD]", 2)
				end
				if yprod>0 then
					city:ChangeProduction(yprod)
					Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(city:Plot():GetX(), city:Plot():GetY()))), "+" .. yprod .. " [ICON_PRODUCTION]", 3)
				end
			end
		end			
	end
end

if JFD_IsCivilisationActive(iCiv) and Game_IsUsingMod(CBPmodID) then 
	GameEvents.UnitPillageGold.Add(OnPillageCaptureTileYield)
end


--=======================================================================================================================
--TROLLER'S ADDITIONAL ACHIEVEMENTS
--=======================================================================================================================
local TAAid = "432bc547-eb05-4189-9e46-232dbde8f09f"
local isAAActive = Game_IsUsingMod(TAAid)
----------------------------------------

function UnlockNavajoAchievement(iPlayer)
	local pPlayer = Players[iPlayer]
	if IsAAUnlocked('AA_JAR_NAVAJO_SPECIAL') then return end
	if (pPlayer:GetCivilizationType() == iCiv and pPlayer:IsAlive()) then
		local citynum = 0
		for city in pPlayer:Cities() do
			if city:GetOriginalOwner() == iPlayer then
				citynum = citynum +1
			end			
		end
		if pPlayer:GetBuildingClassCount(GameInfo.BuildingClasses.BUILDINGCLASS_PUBLIC_SCHOOL.ID) == citynum then
			UnlockAA('AA_JAR_NAVAJO_SPECIAL')
		end
	end
end


if isAAActive then
	GameEvents.PlayerDoTurn.Add(UnlockNavajoAchievement)
end

print("Navajo functions loaded")
--==========================================================================================================================
--==========================================================================================================================